###################################################
### Preliminaries
###################################################

## load packages
library("lavaan")
library("strucchange")
library("mvtnorm")
library("lattice")
library("mirt")
library("caret")

## auxiliary code
source("simpara.R")
source("mz.pml.R")

## data for application
load("irtdata.rda")

###################################################
### Simulation 1
###################################################

## seed for replication
RNGkind(kind = "default", normal.kind = "default")
set.seed(1090)

## run simulation
if(file.exists("sim2.rda")) {
  load("sim2.rda")
} else {
  sim1 <- simulation(sim = "sim1", nobs = c(120,480,960), nrep = 5000,
                     diff = seq(0,4,1), nlevels = c(4, 8, 12),
                     parms = c("alpha", "gamma"), estimator = "PML",
                     meanch=0, meantest=0)
  save(sim1, file="sim2.rda")
}

## arrange results for display
sim1$test <- factor(as.character(sim1$test),
                    levels = c("ordmax","ordwmax","catdiff"),
                    labels = c("maxLM_o", "WDM_o", "LM_uo"))
parlabs <- c(expression(alpha[1]), expression(alpha[2]),
             expression(alpha[3]), expression(alpha[4]),expression(alpha[5]),
             expression(gamma[1]), expression(gamma[2]), expression(gamma[3]),
             expression(gamma[4]), expression(gamma[5]),
             expression(list(alpha[1], ldots, alpha[5])),
             expression(list(gamma[1], ldots, gamma[5])))
levels(sim1$pars) <- c("Alpha1", "Alpha2", "Alpha3", "Alpha4", "Alpha5",
                       "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5",
                       "All Alphas", "All Gammas")
sim1$nlevels <- factor(sim1$nlevels)
levels(sim1$nlevels) <- paste("m=", levels(sim1$nlevels), sep="")
sim1$nobs <- factor(sim1$nobs)
levels(sim1$nobs) <- paste("n=", levels(sim1$nobs), sep="")


###################################################
### Figure 1
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ test, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & nobs == "n=960"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 3, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 2
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ test, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha2", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & nobs =="n=960"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(3, 7, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 3
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$nobs), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ nobs, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & test =="WDM_o"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 3, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 4
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$nobs), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ nobs, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha3", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & test =="WDM_o"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(3, 7, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Simulation 2.1
###################################################
## seed for replication
RNGkind(kind = "default", normal.kind = "default")
set.seed(1090)

## run simulation
if(file.exists("sim8.rda")) {
  load("sim8.rda")
} else {
  sim1 <- simulation(sim = "sim1", nobs = c(1200, 4800, 9600), nrep = 5000,
                     diff = seq(0,4,1), nlevels = 8,
                     parms = c("alpha", "gamma"), estimator = "PML",
                     meanch=0, meantest=2)
  save(sim1, file="sim8.rda")
}

## arrange results for display
sim1$test <- factor(as.character(sim1$test),
                    levels = c("ordmax","ordwmax","catdiff"),
                    labels = c("maxLM_o", "WDM_o", "LM_uo"))
parlabs <- c(expression(alpha[1]), expression(alpha[2]),
             expression(alpha[3]), expression(alpha[4]),expression(alpha[5]),
             expression(gamma[1]), expression(gamma[2]), expression(gamma[3]),
             expression(gamma[4]), expression(gamma[5]),
             expression(list(alpha[1], ldots, alpha[5])),
             expression(list(gamma[1], ldots, gamma[5])))
levels(sim1$pars) <- c("Alpha1", "Alpha2", "Alpha3", "Alpha4", "Alpha5",
                       "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5",
                       "All Alphas", "All Gammas")
sim1$nlevels <- factor(sim1$nlevels)
levels(sim1$nlevels) <- paste("m=", levels(sim1$nlevels), sep="")
sim1$nobs <- factor(sim1$nobs)
levels(sim1$nobs) <- paste("n=", levels(sim1$nobs), sep="")


###################################################
### Figure 5
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nobs + pars, group = ~ test, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4)),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 3, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 6
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nobs + pars, group = ~ test, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha3", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4)),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(3, 7, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Simulation 2.2
###################################################
## seed for replication
RNGkind(kind = "default", normal.kind = "default")
set.seed(1090)

## run simulation
if(file.exists("sim4.rda")) {
  load("sim4.rda")
} else {
  sim1 <- simulation(sim = "sim1", nobs = c(1200, 4800, 9600), nrep = 5000,
                     diff = seq(0,4,1), nlevels = 8,
                     parms = c("alpha", "gamma"), estimator = "PML",
                     meanch=2, meantest=2)
  save(sim1, file="sim4.rda")
}

## arrange results for display
sim1$test <- factor(as.character(sim1$test),
                    levels = c("ordmax","ordwmax","catdiff"),
                    labels = c("maxLM_o", "WDM_o", "LM_uo"))
parlabs <- c(expression(alpha[1]), expression(alpha[2]),
             expression(alpha[3]), expression(alpha[4]),expression(alpha[5]),
             expression(gamma[1]), expression(gamma[2]), expression(gamma[3]),
             expression(gamma[4]), expression(gamma[5]),
             expression(list(alpha[1], ldots, alpha[5])),
             expression(list(gamma[1], ldots, gamma[5])))
levels(sim1$pars) <- c("Alpha1", "Alpha2", "Alpha3", "Alpha4", "Alpha5",
                       "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5",
                       "All Alphas", "All Gammas")
sim1$nlevels <- factor(sim1$nlevels)
levels(sim1$nlevels) <- paste("m=", levels(sim1$nlevels), sep="")
sim1$nobs <- factor(sim1$nobs)
levels(sim1$nobs) <- paste("n=", levels(sim1$nobs), sep="")


###################################################
### Figure 7
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nobs + pars, group = ~ test, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4)),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 3, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 8
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nobs + pars, group = ~ test, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha3", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4)),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(3, 7, 8, 11, 12)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Arrange data for application section
###################################################
ISIlevel <- unique(datstu$ISI)
SESlevel <- unique(datstu$SES)
datstu$total <- apply(datstu[,1:18], 1, mean)
datstu$SESgroup <- rep(NA, nrow = datstu)
datstu$SESgroup[datstu$SES <= -1] <- 1
datstu$SESgroup[datstu$SES > -1 & datstu$SES <= -0.5] <- 2
datstu$SESgroup[datstu$SES > -0.5 & datstu$SES <= 0] <- 3
datstu$SESgroup[datstu$SES > 0 & datstu$SES <= 0.5] <- 4
datstu$SESgroup[datstu$SES > 0.5 & datstu$SES <= 1] <- 5
datstu$SESgroup[datstu$SES > 1 & datstu$SES <= 3] <- 6
datstu$SESgroup <- as.factor(datstu$SESgroup)
datstu <- datstu[order(datstu$SESgroup),]
data <- datstu


###################################################
### Compute various test statistics
###################################################
## obtain critical values
if(file.exists("maxLMo.rda")){
  load("maxLMo.rda")
} else {
  RNGkind(kind = "default", normal.kind = "default")
  set.seed(1090)
  maxLMo <- ordL2BB(data$SESgroup, nproc = 18)
  save(maxLMo, file = "maxLMo.rda")
}
if(file.exists("maxLMo36.rda")){
  load("maxLMo36.rda")
} else {
  RNGkind(kind = "default", normal.kind = "default")
  set.seed(1090)
  maxLMo36 <- ordL2BB(data$SESgroup, nproc = 36)
  save(maxLMo36, file = "maxLMo36.rda")
}
if(file.exists("maxLMo1.rda")){
  load("maxLMo1.rda")
} else {
  RNGkind(kind = "default", normal.kind = "default")
  set.seed(1090)
  maxLMo1 <- ordL2BB(data$SESgroup, nproc = 1)
  save(maxLMo1, file = "maxLMo1.rda")
}
if(file.exists("maxLMo2.rda")){
  load("maxLMo2.rda")
} else {
  RNGkind(kind = "default", normal.kind = "default")
  set.seed(1090)
  maxLMo2 <- ordL2BB(data$SESgroup, nproc = 2)
  save(maxLMo2, file = "maxLMo2.rda")
}
if(file.exists("maxLMo5.rda")){
  load("maxLMo5.rda")
} else {
  RNGkind(kind = "default", normal.kind = "default")
  set.seed(1090)
  maxLMo5 <- ordL2BB(data$SESgroup, nproc = 5)
  save(maxLMo5, file = "maxLMo5.rda")
}

## define and fit models
f05 <- paste(paste("lam", 1:18, "*", names(data)[1:18],
                   sep = ""), collapse = " + ")
f05 <- paste("theta =~ ", f05, sep = "")
f06 <- paste(names(data)[1:18], " | th", 1:18, "*t1",
             sep = "")
f06 <- paste(f06, collapse = " \n ")
f07 <- paste("alpha", 1:18, " := (lam",1:18, ")/sqrt(1-lam",
             1:18, "^2)", sep = "")
f07 <- paste(f07, collapse = " \n ")
f08 <- paste("beta", 1:18, " := (-th", 1:18, ")/sqrt(1-lam",
             1:18, "^2)", sep = "")
f08 <- paste(f08, collapse = " \n ")
f09 <- paste(names(data)[1:18], " ~*~ 1*", names(data)[1:18],
             sep = "", collapse = " \n ")
f10 <- "theta ~ c(0, NA, NA, NA, NA, NA)*1"
f11 <- "theta ~~ c(1, NA, NA, NA, NA, NA)*theta"
f12 <- "theta ~ c(0, NA, NA, NA, NA, NA)*1 + c(NA, 'm', 'm', 'm', 'm', 'm')*1"
f13 <- "theta ~~ c(1, NA, NA, NA, NA, NA)*theta + 
c(NA, 'v', 'v', 'v', 'v', 'v')*theta"
f14 <- "theta ~~ c(1, NA, NA, NA, NA, NA)*theta + 
c(NA, 'v1', 'v1', 'v1', 'v2', 'v2')*theta"
mod <- paste(f05, f06, f07, f08, sep = " \n ")
writeLines(mod, "mod.txt")

modgroup <- paste(f05, f06, f07, f08, f09, f10, f11, sep = " \n ")
writeLines(modgroup, "modgroup.txt")

modmean <-  paste(f05, f06, f07, f08, f09, f10, sep = " \n ")
writeLines(modmean, "modmean.txt")

modgrcons <- paste(f05, f06, f07, f08, f09, f12, f13, sep = " \n ")
writeLines(modgrcons, "modgrcons.txt")

modgrconsfinal <- paste(f05, f06, f07, f08, f09, f10, f14, sep = " \n ")
writeLines(modgrconsfinal, "modgrconsfinal.txt")

if(file.exists("rval.rda")){
  load("rval.rda")
} else {
  rval <- cfa(mod, data=data, std.lv=TRUE,
              ordered=names(data)[1:18], estimator="PML")
  save(rval, file="rval.rda")
}
if(file.exists("rvalgroup.rda")){
  load("rvalgroup.rda")
} else {
  rvalgroup <- cfa(modgroup, data=data, std.lv=TRUE,
                   ordered=names(data)[1:18], group = "SESgroup", group.equal=
                     c("loadings", "thresholds"), estimator="PML")
  save(rvalgroup, file="rvalgroup.rda")
}
if(file.exists("rvalmean.rda")){
  load("rvalmean.rda")
} else {
  rvalmean <- cfa(modmean, data=data, std.lv=TRUE,
                  ordered=names(data)[1:18], group = "SESgroup", group.equal=
                    c("loadings", "thresholds"), estimator="PML")
  save(rvalmean, file="rvalmean.rda")
}
if(file.exists("rvalgrcons.rda")){
  load("rvalgrcons.rda")
} else {
  rvalgrcons <- cfa(modgrcons, data=data, std.lv=TRUE,
                    ordered=names(data)[1:18], group = "SESgroup", group.equal=
                      c("loadings", "thresholds"), estimator="PML")
  save(rvalgrcons, file="rvalgrcons.rda")
}
if(file.exists("rvalgrconsfinal.rda")){
  load("rvalgrconsfinal.rda")
} else {
  rvalgrconsfinal <- cfa(modgrconsfinal, data=data, std.lv=TRUE,
                         ordered=names(data)[1:18], group = "SESgroup", group.equal=
                           c("loadings", "thresholds"), estimator="PML")
  save(rvalgrconsfinal, file="rvalgrconsfinal.rda")
}
WDMoSESdisc <- rep(NA, 18)
WDMoSESdiff <- rep(NA, 18)
WDMoSESgroupdisc <- rep(NA, 18)
WDMoSESgroupdiff <- rep(NA, 18)
maxLMoSESdisc <- rep(NA, 18)
maxLMoSESdiff <- rep(NA, 18)
maxLMoSESgroupdisc <- rep(NA, 18)
maxLMoSESgroupdiff <- rep(NA, 18)

## obtain test statistics/p-values
if(file.exists("parares.rda")){
  load("parares.rda")
} else {
  for (i in 1:18){
    WDMoSESdisc[i] <- sctest(rval, order.by = data$SESgroup, parm = i,
                             vcov = NULL,
                             functional = "WDMo", plot = FALSE)$p.value
    WDMoSESdiff[i] <- sctest(rval, order.by = as.factor(data$SESgroup),
                             parm = i+18, vcov = NULL,
                             functional = "WDMo", plot = FALSE)$p.value
    WDMoSESgroupdisc[i] <- sctest(rvalmean, order.by = data$SESgroup,
                                  parm = i, vcov = NULL,
                                  functional = "WDMo", plot = FALSE)$p.value
    WDMoSESgroupdiff[i] <- sctest(rvalmean, order.by = data$SESgroup,
                                  parm = i+18, vcov = NULL,
                                  functional = "WDMo", plot = FALSE)$p.value
    maxLMoSESdisc[i] <- sctest(rval, order.by = data$SESgroup, parm = i,
                               vcov = NULL,
                               functional = maxLMo1, plot = FALSE)$p.value
    maxLMoSESdiff[i] <- sctest(rval, order.by = as.factor(data$SESgroup),
                               parm = i+18, vcov = NULL,
                               functional = maxLMo1, plot = FALSE)$p.value
    maxLMoSESgroupdisc[i] <- sctest(rvalmean, order.by = data$SESgroup,
                                    parm = i, vcov = NULL,
                                    functional = maxLMo1, plot = FALSE)$p.value
    maxLMoSESgroupdiff[i] <- sctest(rvalmean, order.by = data$SESgroup,
                                    parm = i+18, vcov = NULL,
                                    functional = maxLMo1, plot = FALSE)$p.value
  }
  
  parares <- cbind(WDMoSESdisc, WDMoSESdiff, WDMoSESgroupdisc,
                   WDMoSESgroupdiff, maxLMoSESdisc, maxLMoSESdiff,
                   maxLMoSESgroupdisc, maxLMoSESgroupdiff)
  save(parares, file ="parares.rda")
}

if(file.exists("personparares.rda")){
  load("personparares.rda")
} else {
  personparares <- rep(NA, 3)
  personparares[1] <- sctest(rvalgrcons, order.by = data$SESgroup,
                             parm = 37, vcov = NULL,
                             functional = maxLMo1, plot = TRUE)$p.value
  personparares[2] <- sctest(rvalgrcons, order.by = data$SESgroup,
                             parm = 38, vcov = NULL,
                             functional = maxLMo1, plot = FALSE)$p.value
  personparares[3] <- sctest(rvalgrconsfinal, order.by = data$SESgroup,
                             parm = 38, vcov = NULL,
                             functional = maxLMo1, plot = FALSE)$p.value
  save(personparares, file ="personparares.rda")
}
###################################################
### Overall Model Fit and Cross Validation
###################################################
## overall model fit
if (file.exists("testrvalgrconsfinal.rda") & file.exists("testrvalgrcons.rda") & 
    file.exists("testrvalgroup.rda") & file.exists("plrt1.rda") & file.exists("plrt.rda") & 
    file.exists("plrt2.rda")) {
  load("testrvalgrconsfinal.rda")
  load("testrvalgrcons.rda")
  load("testrvalgroup.rda")
  load("plrt1.rda")
  load("plrt.rda")
  load("plrt2.rda")
} else {
  set.seed(1090)
  testrvalgrconsfinal <- lavaan:::ctr_pml_aic_bic(rvalgrconsfinal)
  testrvalgrcons <- lavaan:::ctr_pml_aic_bic(rvalgrcons)
  testrvalgroup <- lavaan:::ctr_pml_aic_bic(rvalgroup)
  plrt1 <- ctr_pml_plrt_nested(rvalgrconsfinal, rvalgroup)
  plrt <- ctr_pml_plrt_nested(rvalgrcons, rvalgrconsfinal)
  plrt2 <- ctr_pml_plrt_nested(rvalgrcons, rvalgroup)
  ## save results
  save(testrvalgrconsfinal, file = "testrvalgrconsfinal.rda")
  save(testrvalgrcons, file = "testrvalgrcons.rda")
  save(testrvalgroup, file = "testrvalgroup.rda")
  save(plrt1, file = "plrt1.rda")
  save(plrt, file = "plrt.rda")
  save(plrt2, file = "plrt2.rda")
}

## cross validation
if (file.exists("CVPLRTmodgrcons.rda") & file.exists("CVPLRTmodgrconsfinal.rda") & 
    file.exists("CVPLRTmodgroup.rda") & 
    file.exists("CVPLRTmodgrconsfinal_stats.rda") &
    file.exists("CVPLRTmodgrcons_stats.rda") & 
    file.exists("CVPLRTmodgroup_stats.rda") &
    file.exists("testCVPLRTmodgrcons.rda") & 
    file.exists("testCVPLRTmodgrconsfinal.rda") &
    file.exists("testCVPLRTmodgroup.rda") &
    file.exists("testCVPLRTmodgrcons_stats.rda") &
    file.exists("testCVPLRTmodgrconsfinal_stats.rda") &
    file.exists("testCVPLRTmodgroup_stats.rda") &
    file.exists("CVplrt.rda") &
    file.exists("trainCVplrt.rda") &
    file.exists("paraCVPLRTmodgrconsfinal.rda") & 
    file.exists("trainparaCVPLRTmodgrconsfinal.rda")&  
    file.exists("paraCVPLRTmodgrconsfinal2.rda") & 
    file.exists("trainparaCVPLRTmodgrconsfinal2.rda")) {
  load("CVPLRTmodgrcons.rda")
  load("CVPLRTmodgrconsfinal.rda")
  load("CVPLRTmodgroup.rda")
  load("CVPLRTmodgrconsfinal_stats.rda") 
  load("CVPLRTmodgrcons_stats.rda") 
  load("CVPLRTmodgroup_stats.rda")
  load("testCVPLRTmodgrcons.rda")
  load("testCVPLRTmodgrconsfinal.rda")
  load("testCVPLRTmodgroup.rda")
  load("testCVPLRTmodgrcons_stats.rda")
  load("testCVPLRTmodgrconsfinal_stats.rda")
  load("testCVPLRTmodgroup_stats.rda")
  load("CVplrt.rda")
  load("trainCVplrt.rda")
  load("trainparaCVPLRTmodgrconsfinal.rda")
  load("trainparaCVPLRTmodgrconsfinal2.rda")
} else {
  set.seed(1090)
  nrep = 100
  trainIndex <- createDataPartition(as.factor(data$SESgroup), p = .5, 
                                    list = FALSE, 
                                    times = nrep)
  CVPLRTmodgroup <- vector("list", nrep)
  CVPLRTmodgrcons <- vector("list", nrep)
  CVPLRTmodgrconsfinal <- vector("list", nrep)
  CVPLRTmodgroup_stats <- vector("list", nrep)
  CVPLRTmodgrcons_stats <- vector("list", nrep)
  CVPLRTmodgrconsfinal_stats <- vector("list", nrep)
  trainCVplrt <- vector("list", nrep)
  CVplrt <- vector("list", nrep)
  testCVPLRTmodgroup <- vector("list", nrep)
  testCVPLRTmodgrcons <- vector("list", nrep)
  testCVPLRTmodgrconsfinal <- vector("list", nrep)
  testCVPLRTmodgroup_stats <- vector("list", nrep)
  testCVPLRTmodgrcons_stats <- vector("list", nrep)
  testCVPLRTmodgrconsfinal_stats <- vector("list", nrep)
  paraCVPLRTmodgrconsfinal <- vector("list", nrep)
  paraCVPLRTmodgrconsfinal2 <- vector("list", nrep)
  trainparaCVPLRTmodgrconsfinal <- vector("list", nrep)
  trainparaCVPLRTmodgrconsfinal2 <- vector("list", nrep)
  do.parallel <- require("parallel")
  if (do.parallel){
    ## training data: three models
    ## group model
    CVPLRTmodgroup <- mclapply(1:nrep, function(i){
      cfa(modgroup, data = data[trainIndex[,i],], 
          std.lv=TRUE, ordered = names(data)[1:18], 
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator = "PML")}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVPLRTmodgroup, file = "CVPLRTmodgroup.rda")
    ## stats
    CVPLRTmodgroup_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(CVPLRTmodgroup[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVPLRTmodgroup_stats, file = "CVPLRTmodgroup_stats.rda")
    ## grcons model
    CVPLRTmodgrcons <- mclapply(1:nrep, function(i){
      cfa(modgrcons, data = data[trainIndex[,i],], 
          std.lv=TRUE, ordered = names(data)[1:18], 
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator = "PML")}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    
    save(CVPLRTmodgrcons, file = "CVPLRTmodgrcons.rda")
    ## stats
    CVPLRTmodgrcons_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(CVPLRTmodgrcons[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVPLRTmodgrcons_stats, file = "CVPLRTmodgrcons_stats.rda")
    
    CVPLRTmodgrconsfinal <- mclapply(1:nrep, function(i){
      cfa(modgrconsfinal, 
          data = data[trainIndex[,i],], 
          std.lv=TRUE, ordered = names(data)[1:18], 
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator = "PML")},
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVPLRTmodgrconsfinal, file = "CVPLRTmodgrconsfinal.rda")
    ## stats
    CVPLRTmodgrconsfinal_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(CVPLRTmodgrconsfinal[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVPLRTmodgrconsfinal_stats, file = "CVPLRTmodgrconsfinal_stats.rda")
    
    ## compare the top two by using plrt_nested. 
    trainCVplrt <- mclapply(1:nrep, function(i){
      ctr_pml_plrt_nested(CVPLRTmodgroup[[i]], CVPLRTmodgrconsfinal[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(trainCVplrt, file = "trainCVplrt.rda")
    
    ## corresponding test part     
    testCVPLRTmodgrcons <- mclapply(1:nrep, function(i){
      lavoptions <- CVPLRTmodgrcons[[i]]@Options
      lavoptions$se <- "none"
      lavoptions$optim.method <- "none"
      
      cfa(slotParTable = CVPLRTmodgrcons[[i]]@ParTable,
          slotModel = CVPLRTmodgrcons[[i]]@Model,
          data = data[-trainIndex[,i],], 
          slotOptions = lavoptions,
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator= "PML")},
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgrcons, file = "testCVPLRTmodgrcons.rda")
    ## stats
    testCVPLRTmodgrcons_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(testCVPLRTmodgrcons[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgrcons_stats, file = "testCVPLRTmodgroup_stats.rda")
    
    testCVPLRTmodgrconsfinal <- mclapply(1:nrep, function(i){
      lavoptions <- CVPLRTmodgrconsfinal[[i]]@Options
      lavoptions$se <- "none"
      lavoptions$optim.method <- "none"
      
      cfa(slotParTable = CVPLRTmodgrconsfinal[[i]]@ParTable,
          slotModel = CVPLRTmodgrconsfinal[[i]]@Model,
          data = data[-trainIndex[,i],], 
          slotOptions = lavoptions,
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator= "PML")},
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgrconsfinal, file = "testCVPLRTmodgrconsfinal.rda")
    
    ## stats
    testCVPLRTmodgrconsfinal_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(testCVPLRTmodgrconsfinal[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgrconsfinal_stats, 
         file = "testCVPLRTmodgrconsfinal_stats.rda")
    
    testCVPLRTmodgroup <- mclapply(1:nrep, function(i){
      lavoptions <- CVPLRTmodgroup[[i]]@Options
      lavoptions$se <- "none"
      lavoptions$optim.method <- "none"
      
      cfa(slotParTable = CVPLRTmodgroup[[i]]@ParTable,
          slotModel = CVPLRTmodgroup[[i]]@Model,
          data = data[-trainIndex[,i],], 
          slotOptions = lavoptions,
          group = "SESgroup", group.equal =
            c("loadings", "thresholds"), 
          estimator= "PML")},
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgroup, file = "testCVPLRTmodgroup.rda")
    
    ## stats
    testCVPLRTmodgroup_stats <- mclapply(1:nrep, function(i){
      lavaan:::ctr_pml_aic_bic(testCVPLRTmodgroup[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(testCVPLRTmodgroup_stats, file = "testCVPLRTmodgroup_stats.rda")
    
    
    ## compare the top two by using plrt_nested. 
    CVplrt <- mclapply(1:nrep, function(i){
      ctr_pml_plrt_nested(testCVPLRTmodgroup[[i]], testCVPLRTmodgrconsfinal[[i]])}, 
      mc.cores = round(.75*detectCores()), mc.preschedule = FALSE)
    save(CVplrt, file = "CVplrt.rda")
    
    ## score-based tests on item parameter parts
    trainparaCVPLRTmodgrconsfinal <- mclapply(1:nrep, function(i){
      sctest(CVPLRTmodgrconsfinal[[i]], 
             order.by = data[trainIndex[,i],]$SESgroup, 
             parm = 1:36, vcov = NULL, 
             functional = "WDMo", 
             plot = FALSE)},
      mc.cores = round(.75*detectCores()), 
      mc.preschedule = FALSE)
    save(trainparaCVPLRTmodgrconsfinal, 
         file = "trainparaCVPLRTmodgrconsfinal.rda")
    
    trainparaCVPLRTmodgrconsfinal2 <- mclapply(1:nrep, function(i){
      sctest(CVPLRTmodgrconsfinal[[i]], 
             order.by = data[trainIndex[,i],]$SESgroup, 
             parm = 1:36, vcov = NULL, 
             functional = maxLMo36, 
             plot = FALSE)},
      mc.cores = round(.75*detectCores()), 
      mc.preschedule = FALSE)
    save(trainparaCVPLRTmodgrconsfinal2, 
         file = "trainparaCVPLRTmodgrconsfinal2.rda")
    
  }
} 
## organize cross validation results
traincrosspvalue <- unlist(trainparaCVPLRTmodgrconsfinal)[seq(2, 400, 4)]
crosspvalue <- unlist(paraCVPLRTmodgrconsfinal)[seq(2, 400, 4)]
traincrosspvalue2 <- unlist(trainparaCVPLRTmodgrconsfinal2)[seq(2, 400, 4)]
crosspvalue2 <- unlist(paraCVPLRTmodgrconsfinal2)[seq(2, 400, 4)]
trainorderingaic <- sum(unlist(CVPLRTmodgrcons_stats)[seq(2, 300, 3)] > 
                          unlist(CVPLRTmodgroup_stats)[seq(2, 300, 3)] & 
                          unlist(CVPLRTmodgroup_stats)[seq(2, 300, 3)] > 
                          unlist(CVPLRTmodgrconsfinal_stats)[seq(2, 300, 3)])

trainorderingbic <- sum(unlist(CVPLRTmodgrcons_stats)[seq(3, 300, 3)] > 
                          unlist(CVPLRTmodgroup_stats)[seq(3, 300, 3)] & 
                          unlist(CVPLRTmodgroup_stats)[seq(3, 300, 3)] > 
                          unlist(CVPLRTmodgrconsfinal_stats)[seq(3, 300, 3)])

testorderingaic <- sum(unlist(testCVPLRTmodgrcons_stats)[seq(2, 300, 3)] > 
                         unlist(testCVPLRTmodgroup_stats)[seq(2, 300, 3)] & 
                         unlist(testCVPLRTmodgroup_stats)[seq(2, 300, 3)] > 
                         unlist(testCVPLRTmodgrconsfinal_stats)[seq(2, 300, 3)])

testorderingbic <- sum(unlist(testCVPLRTmodgrcons_stats)[seq(3, 300, 3)] > 
                         unlist(testCVPLRTmodgroup_stats)[seq(3, 300, 3)] & 
                         unlist(testCVPLRTmodgroup_stats)[seq(3, 300, 3)] > 
                         unlist(testCVPLRTmodgrconsfinal_stats)[seq(3, 300, 3)])


###################################################
### Appendix A: Results for MML
###################################################
## ltm
library("ltm")
library("strucchange")
library("mvtnorm")
library("lattice")
source("mz.mml.R")

## seed for replication
RNGkind(kind = "default", normal.kind = "default")
set.seed(1090)

## run simulation
if(file.exists("sim1.rda")) {
  load("sim1.rda")
} else {
  sim1 <- simulation(sim = "sim1", nobs = c(120,480,960), nrep = 5000,
                     diff = seq(0,4,1), nlevels = c(4, 8, 12),
                     parms = c("alpha", "gamma"), estimator = "MML",
                     meanch=0, meantest=0)
  save(sim1, file="sim1.rda")
}
sim1$test <- factor(as.character(sim1$test),
                    levels = c("ordmax","ordwmax","catdiff"),
                    labels = c("maxLM_o", "WDM_o", "LM_uo"))
parlabs <- c(expression(alpha[1]), expression(alpha[2]),
             expression(alpha[3]), expression(alpha[4]),expression(alpha[5]),
             expression(gamma[1]), expression(gamma[2]), expression(gamma[3]),
             expression(gamma[4]), expression(gamma[5]),
             expression(list(alpha[1], ldots, alpha[5])),
             expression(list(gamma[1], ldots, gamma[5])))
levels(sim1$pars) <- c("Alpha1", "Alpha2", "Alpha3", "Alpha4", "Alpha5",
                       "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5",
                       "All Alphas", "All Gammas")
sim1$nlevels <- factor(sim1$nlevels)
levels(sim1$nlevels) <- paste("m=", levels(sim1$nlevels), sep="")
sim1$nobs <- factor(sim1$nobs)
levels(sim1$nobs) <- paste("n=", levels(sim1$nobs), sep="")


###################################################
### Figure 11
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ test, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & nobs == "n=960"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 8, 3, 12, 11)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 12
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$test), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ test, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha3", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & nobs =="n=960"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(8, 3, 7, 12, 11)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 13
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$nobs), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ nobs, data = sim1,
       subset = (parms == "alpha" &
                   pars %in% c("Alpha2", "Alpha3", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & test =="WDM_o"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", alpha[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(2, 8, 3, 12, 11)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })


###################################################
### Figure 14
###################################################
trellis.par.set(theme = canonical.theme(color = FALSE))
mykey <- simpleKey(levels(sim1$nobs), points = TRUE, lines = TRUE)
xyplot(power ~ diff | nlevels + pars, group = ~ nobs, data = sim1,
       subset = (parms == "gamma" &
                   pars %in% c("Alpha3", "Gamma2", "Gamma3",
                               "All Alphas", "All Gammas") &
                   diff %in% c(0,1,2,3,4) & test =="WDM_o"),
       type = "b",
       xlab = expression(paste("Violation Magnitude (", gamma[3], ")")),
       ylab = "Power", key = mykey, as.table = TRUE,
       strip = function(..., which.given, factor.levels){
         if(which.given == 2){
           strip.default(which.given, factor.levels =
                           parlabs[c(8, 3, 7, 12, 11)], ...)
         } else {
           strip.default(which.given, factor.levels =
                           factor.levels, ...)
         }
       })
